//////////////////////////////////////////////////////////////////////////////
//
//  Xilinx, Inc. 2002                 www.xilinx.com
//
//  XAPP 268 - Automatic Phase Alignment
//
//////////////////////////////////////////////////////////////////////////////
//
//  File name :       bcd35_half_range.v
//
//  Description :     3-1/2 decade bcd up/down counter with synchronous enable
//                    and asynchronous clear (to 0)
//
//  Date - revision : 20th May 2003 - v 2.1
//
//  Author :          NJS
//
//  Contact : e-mail  hotline@xilinx.com
//            phone   + 1 800 255 7778
//
//  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
//              provided to you "as is". Xilinx and its licensors make, and you
//              receive no warranties or conditions, express, implied,
//              statutory or otherwise, and Xilinx specifically disclaims any
//              implied warranties of merchantability, non-infringement, or
//              fitness for a particular purpose. Xilinx does not warrant that
//              the functions contained in these designs will meet your
//              requirements, or that the operation of these designs will be
//              uninterrupted or error free, or that defects in the Designs
//              will be corrected. Furthermore, Xilinx does not warrant or
//              make any representations regarding use or the results of the
//              use of the designs in terms of correctness, accuracy,
//              reliability, or otherwise.
//
//              LIMITATION OF LIABILITY. In no event will Xilinx or its
//              licensors be liable for any loss of data, lost profits, cost
//              or procurement of substitute goods or services, or for any
//              special, incidental, consequential, or indirect damages
//              arising from the use or operation of the designs or
//              accompanying documentation, however caused and on any theory
//              of liability. This limitation will apply even if Xilinx
//              has been advised of the possibility of such damage. This
//              limitation shall apply not-withstanding the failure of the
//              essential purpose of any limited remedies herein.
//
//  Copyright  2002 Xilinx, Inc.
//  All rights reserved
//
//////////////////////////////////////////////////////////////////////////////
//
`timescale 1 ps / 1ps

module bcd35_half_range (
	clk,				// clock
	rst,				// reset
	enablein,			// enable in
	up,				// count up (not) down
	bcdvec);			// 3 1/2 decade output

input 		clk, rst, enablein, up ;
output [12:0] 	bcdvec ;


wire 		low  ;
wire 		high  ;
reg 	[12:0] 	bcdint  ;

assign low = 1'b0 ;
assign high = 1'b1 ;
assign bcdvec = bcdint ;

always @ (posedge clk or posedge rst)
begin
if (rst)
	bcdint <= 13'b0000000000000;
else begin
	if ((enablein) && (up) && (bcdint != 13'b0001001010101)) begin				// count up
		if ((~bcdint[12]) & (bcdint[7:0] == 8'b10011001))				// +x99 ?
			bcdint <= {bcdint[12], bcdint[11:8] + 4'b0001, 8'b00000000} ;
		else if ((~bcdint[12]) & (bcdint[3:0] == 4'b1001)) 				// +xx9 ?
			bcdint <= {bcdint[12], bcdint[11:8], bcdint[7:4] + 4'b0001, 4'b0000} ;
		else
			bcdint <= {bcdint[12], bcdint[11:4], bcdint[3:0] + 4'b0001} ;
	end
	else begin
	if ((enablein) && (~up) && (bcdint != 13'b0000000000000)) begin				// count down
		if ((~bcdint[12]) & (bcdint[7:0] == 8'b00000000)) 				// +x00 ?
			bcdint <= {bcdint[12], bcdint[11:8] - 4'b0001, 8'b10011001} ;
		else if ((~bcdint[12]) & (bcdint[3:0] == 4'b0000)) 				// + xx0 ?
			bcdint <= {bcdint[12], bcdint[11:8], bcdint[7:4] - 4'b0001, 4'b1001} ;
		else
			bcdint <= {bcdint[12], bcdint[11:4], bcdint[3:0] - 4'b0001} ;
	end
	end
end
end
endmodule


