//////////////////////////////////////////////////////////////////////////////
//
//  Xilinx, Inc. 2002                 www.xilinx.com
//
//  XAPP 268 - Automatic Phase Alignment
//
//////////////////////////////////////////////////////////////////////////////
//
//  File name :       top_full_range.v
//
//  Description :     example top level module using full DCM range
//
//  Date - revision : 20th May 2003 - v 2.1
//
//  Author :          NJS
//
//  Contact : e-mail  hotline@xilinx.com
//            phone   + 1 800 255 7778
//
//  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
//              provided to you "as is". Xilinx and its licensors make, and you
//              receive no warranties or conditions, express, implied,
//              statutory or otherwise, and Xilinx specifically disclaims any
//              implied warranties of merchantability, non-infringement, or
//              fitness for a particular purpose. Xilinx does not warrant that
//              the functions contained in these designs will meet your
//              requirements, or that the operation of these designs will be
//              uninterrupted or error free, or that defects in the Designs
//              will be corrected. Furthermore, Xilinx does not warrant or
//              make any representations regarding use or the results of the
//              use of the designs in terms of correctness, accuracy,
//              reliability, or otherwise.
//
//              LIMITATION OF LIABILITY. In no event will Xilinx or its
//              licensors be liable for any loss of data, lost profits, cost
//              or procurement of substitute goods or services, or for any
//              special, incidental, consequential, or indirect damages
//              arising from the use or operation of the designs or
//              accompanying documentation, however caused and on any theory
//              of liability. This limitation will apply even if Xilinx
//              has been advised of the possibility of such damage. This
//              limitation shall apply not-withstanding the failure of the
//              essential purpose of any limited remedies herein.
//
//  Copyright  2002 Xilinx, Inc.
//  All rights reserved
//
//////////////////////////////////////////////////////////////////////////////
//
`timescale 1 ps / 1ps

module top_full_range (
	rxclkin,		// clock to be monitored
	rst,			// reset
	binout,			// binary output value
	bcdout);		// BCD output value

input 		rxclkin, rst ;
output 	[39:0] 	bcdout ;
output 	[8:0] 	binout ;

// wires

wire 		low  ;
wire 		high ;
wire 		psen ;
wire 		psclk ;
wire 		psdone ;
wire 		psincdec ;
wire 		rxclk ;
wire 		rxclkdcm ;
wire 		notrxclk ;
wire 		notrxclkdcm ;
wire 		dcmlocked ;
wire 		dpclocked ;
wire 		ctlclk ;
wire	[6:0]	const_offset ;
wire 		dcm_div_2_used ;

assign low = 1'b0 ;
assign high = 1'b1 ;
assign const_offset = 7'b0000000 ;				// set up constant offset
assign dcm_div_2_used = 1'b0 ; 					// set up signal to indicate if divide_by_2 is used

// synopsys translate_off //
// Attributes for RTL simulation
defparam dcm_rxclk.CLKOUT_PHASE_SHIFT = "VARIABLE" ;
defparam dcm_rxclk.PHASE_SHIFT = -255 ;
defparam dcm_rxclk.CLKDV_DIVIDE = 4.0 ;
defparam dcm_rxclk.DLL_FREQUENCY_MODE = "HIGH" ;
defparam dcm_rxclk.CLKIN_DIVIDE_BY_2 = "FALSE" ;
// synopsys translate_on

DCM dcm_rxclk (
	.CLKIN(rxclkint),
	.CLKFB(rxclk),
	.DSSEN(low),
	.PSINCDEC(psincdec),
	.PSEN(psen),
	.PSCLK(psclk),
	.RST(rst),
	.CLK0(rxclkdcm),
	.CLKDV(ctlclk),
	.LOCKED(dcmlocked),
	.PSDONE(psdone))

	/*synthesis CLKFX_DIVIDE = 2 CLKFX_MULTIPLY = 1 DUTY_CYCLE_CORRECTION = "TRUE"
	  DFS_FREQUENCY_MODE = "HIGH" CLKOUT_PHASE_SHIFT = "VARIABLE" PHASE_SHIFT = -255
	  CLKDV_DIVIDE = "4.0" DLL_FREQUENCY_MODE = "HIGH" CLKIN_DIVIDE_BY_2 = "FALSE" */ 	// Synplicity attributes
	/*synthesis attribute CLKFX_DIVIDE of dcm_rxclk "2" */					// XST attributes
	/*synthesis attribute CLKFX_MULTIPLY of dcm_rxclk "1" */
	/*synthesis attribute DUTY_CYCLE_CORRECTION of dcm_rxclk TRUE */
	/*synthesis attribute DFS_FREQUENCY_MODE of dcm_rxclk HIGH */
	/*synthesis attribute CLKOUT_PHASE_SHIFT of dcm_rxclk VARIABLE */
	/*synthesis attribute PHASE_SHIFT of dcm_rxclk "-255" */
	/*synthesis attribute CLKDV_DIVIDE of dcm_rxclk "4" */
	/*synthesis attribute CLKIN_DIVIDE_BY_2 of dcm_rxclk FALSE */
	/*synthesis attribute DLL_FREQUENCY_MODE of dcm_rxclk HIGH */  ;

IBUFG rxclk_ibufg    (.I(rxclkin), .O(rxclkint) ) ;

BUFG rxclk_bufg (.I(rxclkdcm),    .O(rxclk) ) ;

phase_control_full_range c0 (
	.enable(high),
	.monclkin(rxclkint),
	.rxclkin(rxclk),
	.ctlclk(ctlclk),
	.lockedin(dcmlocked),
	.rst(rst),
	.psdone(psdone),
	.forceup(low),
	.forcedown(low),
	.dcm_div_2_used(dcm_div_2_used),
	.k(const_offset),
	.lockedout(dpclocked),
	.psincdec(psincdec),
	.psen(psen),
	.psclk(psclk),
	.binout(binout),
	.bcdout(bcdout));

endmodule


