------------------------------------------------------------------------------
--
--  Xilinx, Inc. 2002                 www.xilinx.com
--
--  XAPP 268 - Automatic Phase Alignment
--
------------------------------------------------------------------------------
--
--  File name :       bcd35_half_range.vhd
--
--  Description :     3-1/2 decade bcd up/down counter with synchronous enable
--                    and asynchronous clear (to +000)
--
--  Date - revision : 20th May 2003 - v 2.1
--
--  Author :          NJS
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
--              provided to you "as is". Xilinx and its licensors make, and you
--              receive no warranties or conditions, express, implied,
--              statutory or otherwise, and Xilinx specifically disclaims any
--              implied warranties of merchantability, non-infringement, or
--              fitness for a particular purpose. Xilinx does not warrant that
--              the functions contained in these designs will meet your
--              requirements, or that the operation of these designs will be
--              uninterrupted or error free, or that defects in the Designs
--              will be corrected. Furthermore, Xilinx does not warrant or
--              make any representations regarding use or the results of the
--              use of the designs in terms of correctness, accuracy,
--              reliability, or otherwise.
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its
--              licensors be liable for any loss of data, lost profits, cost
--              or procurement of substitute goods or services, or for any
--              special, incidental, consequential, or indirect damages
--              arising from the use or operation of the designs or
--              accompanying documentation, however caused and on any theory
--              of liability. This limitation will apply even if Xilinx
--              has been advised of the possibility of such damage. This
--              limitation shall apply not-withstanding the failure of the
--              essential purpose of any limited remedies herein.
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved
--
------------------------------------------------------------------------------
--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all ;

--pragma translate_off
library unisim ;
use unisim.vcomponents.all ;
--pragma translate_on

entity bcd35_half_range is port (
	clk		: in std_logic;				-- clock
	rst		: in std_logic;				-- reset
	enablein	: in std_logic;				-- enable in
	up		: in std_logic;				-- count up (not) down
	bcdvec		: out std_logic_vector(12 downto 0));	-- 3 1/2 decade output
end bcd35_half_range;

architecture arch_bcd35_half_range of bcd35_half_range is

signal low 		: std_logic ;
signal high 		: std_logic ;
signal bcdint 		: std_logic_vector(12 downto 0) ;

begin

low 	<= '0' ;
high 	<= '1' ;
bcdvec  <= bcdint ;

process (clk, rst)
begin
if rst = '1' then
	bcdint <= "0000000000000" ;
elsif clk'event and clk = '1' then
	if (enablein = '1' and up = '1' and (bcdint /= "0001001010101")) then			-- count up
		if bcdint(12) = '0' and bcdint(7 downto 0) = "10011001" then			-- +x99 ?
			bcdint <= bcdint(12) & bcdint(11 downto 8) + 1 & "00000000" ;
		elsif bcdint(12) = '0' and bcdint(3 downto 0) = "1001" then			-- +xx9 ?
			bcdint <= bcdint(12) & bcdint(11 downto 8) & bcdint(7 downto 4) + 1 & "0000" ;
		else
			bcdint <= bcdint(12) & bcdint(11 downto 4) & bcdint(3 downto 0) + 1 ;
		end if ;
	elsif (enablein = '1' and up = '0'and (bcdint /= "0000000000000")) then		-- count down
		if bcdint(12) = '0' and bcdint(7 downto 0) = "00000000" then			-- +x00 ?
			bcdint <= bcdint(12) & bcdint(11 downto 8) - 1 & "10011001" ;
		elsif bcdint(12) = '0' and bcdint(3 downto 0) = "0000" then			-- + xx0 ?
			bcdint <= bcdint(12) & bcdint(11 downto 8) & bcdint(7 downto 4) - 1 & "1001" ;
		else
			bcdint <= bcdint(12) & bcdint(11 downto 4) & bcdint(3 downto 0) - 1 ;
		end if ;
	end if ;
end if ;
end process ;
end arch_bcd35_half_range;


