------------------------------------------------------------------------------
--
--  Xilinx, Inc. 2002                 www.xilinx.com
--
--  XAPP 268 - Automatic Phase Alignment
--
------------------------------------------------------------------------------
--
--  File name :       top_full_range.vhd
--
--  Description :     example top level module using full DCM range
--
--  Date - revision : 20th May 2003 - v 2.1
--
--  Author :          NJS
--
--  Contact : e-mail  hotline@xilinx.com
--            phone   + 1 800 255 7778
--
--  Disclaimer: LIMITED WARRANTY AND DISCLAMER. These designs are
--              provided to you "as is". Xilinx and its licensors make, and you
--              receive no warranties or conditions, express, implied,
--              statutory or otherwise, and Xilinx specifically disclaims any
--              implied warranties of merchantability, non-infringement, or
--              fitness for a particular purpose. Xilinx does not warrant that
--              the functions contained in these designs will meet your
--              requirements, or that the operation of these designs will be
--              uninterrupted or error free, or that defects in the Designs
--              will be corrected. Furthermore, Xilinx does not warrant or
--              make any representations regarding use or the results of the
--              use of the designs in terms of correctness, accuracy,
--              reliability, or otherwise.
--
--              LIMITATION OF LIABILITY. In no event will Xilinx or its
--              licensors be liable for any loss of data, lost profits, cost
--              or procurement of substitute goods or services, or for any
--              special, incidental, consequential, or indirect damages
--              arising from the use or operation of the designs or
--              accompanying documentation, however caused and on any theory
--              of liability. This limitation will apply even if Xilinx
--              has been advised of the possibility of such damage. This
--              limitation shall apply not-withstanding the failure of the
--              essential purpose of any limited remedies herein.
--
--  Copyright  2002 Xilinx, Inc.
--  All rights reserved
--
------------------------------------------------------------------------------
--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all ;

--pragma translate_off
library unisim ;
use unisim.vcomponents.all ;
--pragma translate_on

entity top_full_range is port (
	rxclkin		: in std_logic;				-- clock to be monitored
	rst		: in std_logic;				-- reset
	binout		: out std_logic_vector(8 downto 0);	-- binary output value
	bcdout		: out std_logic_vector(39 downto 0));	-- BCD output value
end top_full_range ;
architecture arch_top_full_range of top_full_range is

-- components

-- BUFG primitive
component bufg port(i : in std_logic ; o : out std_logic) ; end component;
-- IBUFG primitive
component ibufg port(i : in std_logic ; o : out std_logic) ; end component;
-- controller
component phase_control_full_range port (
	enable		: in std_logic;				-- enable input, normally high, but allows function to be bypassed to speed up simulation when low
	monclkin	: in std_logic;				-- clock to be monitored
	rxclkin		: in std_logic;				-- clock from dcm
	ctlclk		: in std_logic;				-- phase_control_full_range clock
	lockedin	: in std_logic;				-- dcm locked in
	rst		: in std_logic;				-- reset
	psdone		: in std_logic;				-- high when dcm has changed phase
	forceup		: in std_logic;				-- forces phase value up when high
	forcedown	: in std_logic;				-- forces phase value down when high
	dcm_div_2_used	: in std_logic;				-- normally '0', set to '1' if DCM divide_by_2 option
								-- is used (SDR designs)
	k 		: in std_logic_vector(6 downto 0);	-- constant to be modified in the case where the data
								-- and clock do not arrive at the part exactly aligned
								-- permissable values 0 to 127 (0 to 180 degrees),
								-- set default to "0000000"
	lockedout	: out std_logic;			-- locked out
	psincdec	: out std_logic ;			-- move phase up (not)down
	psen		: out std_logic ;			-- phase shift enable
	psclk		: out std_logic ;			-- phase shift clock
	binout		: out std_logic_vector(8 downto 0);	-- binary current phase  value (0 to 511)
	bcdout		: out std_logic_vector(39 downto 0));	-- 12 downto 0  : BCD current phase value ( -255 to +255)
								-- 26 downto 14 : BCD for ps0
								-- 39 downto 27 : BCD for ps1
end component ;
-- DCM
component dcm
--pragma translate_off
	generic (
	CLKFX_MULTIPLY 		: integer ;
	CLKFX_DIVIDE 		: integer ;
	CLKDV_DIVIDE		: real ;
	CLKOUT_PHASE_SHIFT 	: string ;
	PHASE_SHIFT 		: integer ;
	CLKIN_DIVIDE_BY_2 	: boolean ) ;
--pragma translate_on
	port (
	clkin   	: in  std_logic ;
	CLKFB   	: in  std_logic ;
	DSSEN 		: in  std_logic ;
	PSINCDEC	: in  std_logic ;
	PSEN 		: in  std_logic ;
	PSCLK 		: in  std_logic ;
	RST     	: in  std_logic ;
	CLK0    	: out std_logic ;
	CLK90   	: out std_logic ;
	CLK180  	: out std_logic ;
	CLK270  	: out std_logic ;
	CLK2X   	: out std_logic ;
	CLK2X180	: out std_logic ;
	CLKDV   	: out std_logic ;
	CLKFX   	: out std_logic ;
	CLKFX180	: out std_logic ;
	LOCKED  	: out std_logic ;
	PSDONE  	: out std_logic ;
	STATUS  	: out std_logic_vector(7 downto 0));
end component;

-- signals

signal low 		: std_logic ;
signal high 		: std_logic ;
signal psen 		: std_logic ;
signal psclk 		: std_logic ;
signal psdone 		: std_logic ;
signal psincdec 	: std_logic ;
signal rxclk 		: std_logic ;
signal rxclkdcm 	: std_logic ;
signal notrxclk 	: std_logic ;
signal dcmlocked 	: std_logic ;
signal dpclocked 	: std_logic ;
signal ctlclk 		: std_logic ;
signal rxclkint 	: std_logic ;
signal dcm_div_2_used 	: std_logic ;
signal const_offset 	: std_logic_vector(6 downto 0) ;

-- attributes

attribute DLL_FREQUENCY_MODE 	: string;
attribute DFS_FREQUENCY_MODE 	: string;
attribute CLKOUT_PHASE_SHIFT 	: string;
attribute PHASE_SHIFT 		: string;
attribute CLKFX_DIVIDE 		: string;
attribute CLKFX_MULTIPLY 	: string;
attribute CLKDV_DIVIDE 		: string;
attribute DUTY_CYCLE_CORRECTION : string;
attribute CLKIN_DIVIDE_BY_2 	: string;

attribute DLL_FREQUENCY_MODE 	of dcm_rxclk: label is "HIGH" ;
attribute CLKOUT_PHASE_SHIFT 	of dcm_rxclk: label is "VARIABLE" ;	-- set up the dcm in variable mode
attribute PHASE_SHIFT        	of dcm_rxclk: label is "-255" ;		-- with initial value of -255
attribute CLKFX_DIVIDE       	of dcm_rxclk: label is "2" ;
attribute CLKFX_MULTIPLY     	of dcm_rxclk: label is "1" ;
attribute DUTY_CYCLE_CORRECTION of dcm_rxclk: label is "TRUE" ;
attribute CLKDV_DIVIDE          of dcm_rxclk: label is "4" ;
attribute CLKIN_DIVIDE_BY_2 	of dcm_rxclk: label is "FALSE" ;

begin

low 	<= '0' ;
high 	<= '1' ;
dcm_div_2_used <= '0' ; 						-- set up signal to indicate if divide_by_2 is used
--const_offset <= "0000000" ;						-- set up constant offset
const_offset <= "0110011" ;						-- advance by 51/256 for test
--const_offset <= "1100100" ;						-- advance by 100/256 for test

dcm_rxclk : dcm
--pragma translate_off
	generic map (
	CLKFX_MULTIPLY 		=> 1,
	CLKFX_DIVIDE		=> 2,
	CLKDV_DIVIDE		=> 4.0,
	CLKOUT_PHASE_SHIFT 	=> "VARIABLE",
	PHASE_SHIFT 		=> -255,
	CLKIN_DIVIDE_BY_2 	=> FALSE)
--pragma translate_on
	port map (
	clkin   	=> rxclkint,
	CLKFB   	=> rxclk,
	DSSEN 		=> low,
	PSINCDEC	=> psincdec,
	PSEN 		=> psen,
	PSCLK 		=> psclk,
	RST     	=> rst,
	CLK0    	=> rxclkdcm,
	CLK90   	=> open,
	CLK180  	=> open,
	CLK270  	=> open,
	CLK2X   	=> open,
	CLK2X180	=> open,
	CLKDV   	=> ctlclk,
	CLKFX   	=> open,
	CLKFX180	=> open,
	LOCKED  	=> dcmlocked,
	PSDONE  	=> psdone,
	STATUS  	=> open);

rxclk_ibufg 	: ibufg port map(i => rxclkin,    o => rxclkint ) ;

rxclk_bufg 	: bufg 	port map(i => rxclkdcm,    o => rxclk ) ;

c0 : phase_control_full_range port map(
	enable		=> high,
	monclkin	=> rxclkint,
	rxclkin		=> rxclk,
	ctlclk		=> ctlclk,
	lockedin	=> dcmlocked,
	rst		=> rst,
	psdone		=> psdone,
	forceup		=> low,
	forcedown	=> low,
	dcm_div_2_used	=> dcm_div_2_used,
	k 		=> const_offset,
	lockedout	=> dpclocked,
	psincdec	=> psincdec,
	psen		=> psen,
	psclk		=> psclk,
	binout		=> binout,
	bcdout		=> bcdout);

end arch_top_full_range;


